import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;

import org.snia.xam.XAMException;
import org.snia.xam.XAMLibrary;
import org.snia.xam.XSet;
import org.snia.xam.XStream;
import org.snia.xam.XSystem;
import org.snia.xam.XUID;
import org.snia.xam.util.XAMLibraryFactory;


/******************************************************************************
*
* Copyright (c) 2008-2009 EMC Corporation
* All Rights Reserved
*
* StoreContentWithRetention.java
*
* Sample program of XAM to store content on XSystem
*
* This source file contains the intellectual property of EMC Corporation
* or is licensed to EMC Corporation from third parties. Use of this source file
* and the intellectual property contained therein is expressly limited to the
* terms and conditions of the License Agreement.
*
******************************************************************************/

public class StoreContentWithRetention {
	private static XAMLibrary xamLib;
	private static XSystem xSystem;
	private static String appName = "XAM Store Content Sample";
	private static String appVendor = "EMC";
	private static String appVersion = "1.0";
	private static final int BUFFER_SIZE = 1024 * 1024;

	private static void saveIDToFile(XUID xuID, String filename) {
		String xuIDFilename = filename;
		xuIDFilename += ".XUID";

		try {
			FileWriter clipIDWriter = new FileWriter("Output/"+xuIDFilename);
			clipIDWriter.write(xuID.toString());
			clipIDWriter.close();
			System.out.println(
				"XUID of XSet written to file \""
					+ xuIDFilename
					+ "\".");
		} catch (IOException e) {
			throw new IllegalArgumentException(
				"Problem saving XUID to file \""
					+ xuIDFilename
					+ "\"\nObject successfully stored as XUID \""
					+ xuID);
		}
	}

	private static XUID storeFile(String filename,String retentionType,String retentionDuration) throws XAMException {
		XUID xuID = null;
		long totalRead = 0;
		long totalWrite = 0;

		try {
			BufferedInputStream inputStream = new BufferedInputStream(new FileInputStream(new File("Input/"+filename)));

			System.out.println("Create XSet");
			XSet xSet = xSystem.createXSet(XSet.MODE_UNRESTRICTED);

			System.out.println("Set application properties");
			xSet.createProperty("app-vendor", false, appVendor);
			xSet.createProperty("app-name", false, appName);
			xSet.createProperty("app-version", false, appVersion);

			if(retentionType.equalsIgnoreCase("Event")) {

				 xSet.createRetention(true, "event");
				 xSet.setRetentionEnabledFlag("event", false, true);
				 xSet.setRetentionDuration("event", false, Long.parseLong(retentionDuration));
				// xSet.setRetentionStarttime("event", false);
				xSet.setBaseRetention(false,0);

			}else if(retentionType.equalsIgnoreCase("Base")) {
					xSet.setBaseRetention(true, Long.parseLong(retentionDuration));
			}


			//System.out.println("Create XStream");
			XStream xStream = xSet.createXStream("stream", true, XAMLibrary.DEFAULT_MIME_TYPE);

			byte[] buffer = new byte[BUFFER_SIZE];
			long bytesRead = 0;

			while((bytesRead = inputStream.read(buffer)) > 0) {
				long bytesPos = 0;
				while(bytesPos < bytesRead) {
					long bytesWrite = xStream.write(buffer, bytesPos, bytesRead - bytesPos);
					System.out.println("XStream Write: " + bytesWrite);
					bytesPos += bytesWrite;
				}
				totalRead += bytesRead;
				totalWrite += bytesPos;
			}
			inputStream.close();

			//System.out.println("Total Bytes Read From File: " + totalRead);
			System.out.println("Total Bytes Written to XStream: " + totalWrite);

			System.out.println("Set filename property");
			xSet.createProperty("filename", true, filename);

			xStream.close();

			//System.out.println("XSet Commit");
			xuID = xSet.commit();
			//System.out.println("Returned XUID: " + xuID.toString());

			xSet.close();
		} catch (FileNotFoundException fe) {
			throw new IllegalArgumentException(
				"Could not open file \"" + filename + "\" for reading");
		} catch (IOException ioe) {
			System.err.println("Error reading from file \"" + filename + "\"");
			ioe.printStackTrace();
		}catch(Exception e){
			e.printStackTrace();
		}

		return (xuID);
	}

	public static void main(String[] args) {
		//String xri = "snia-xam://centera_vim!128.221.200.60?us2_armTest1.pea";
		//String xri = "snia-xam://SNIA_Reference_VIM!localhost?dir=C:\\xam_storage";
		String xri = "snia-xam://centera_vim!167.113.250.163?TPC-FP-imgtdb03.pea";
		long exitCode = 0;

		InputStreamReader inputReader = new InputStreamReader(System.in);
		BufferedReader stdin = new BufferedReader(inputReader);

		try {
			xamLib = XAMLibraryFactory.newXAMLibrary();
			xamLib.createProperty("com.emc.centera.application.name", false, appName);
			xamLib.createProperty("com.emc.centera.application.version", false, appVersion);

			//System.out.print("Address of XSystem[" + xri + "]: ");
			//String answer = stdin.readLine();

			//if (!answer.equals(""))
			//	xri = answer;

			System.out.println("Connecting to XSystem(" + xri + ")");

			xSystem = xamLib.connect(xri);

			System.out.print("Filename to store: ");

			String filename = stdin.readLine();
			if (filename.equals(""))
				throw new IllegalArgumentException("Invalid answer.");

			//System.out.print("Select Retention Type, Base or Event : ");
			//String retType = stdin.readLine();
			//if (!retType.equalsIgnoreCase("Event") && !retType.equalsIgnoreCase("Base"))
				//throw new IllegalArgumentException("Invalid answer.");

				String retType="Event";
				String retDuration="1000";

			//System.out.print("Select Retention Duartion ");
			//String setRet = stdin.readLine();

			//filename ="../Document Input/PDF/"+filename;
			//XUID xuID = storeFile(filename,retType,setRet);
			XUID xuID = storeFile(filename,retType,retDuration);
			saveIDToFile(xuID, filename);

			xSystem.close();
			System.out.println("\nClosed connection to XSystem (" + xri + ")");

			inputReader.close();
			stdin.close();

		} catch (XAMException xe) {
			exitCode = xe.getStatusCode();
			System.err.println(
				"XAM Error occured: " + xe.getMessage() + "(" + exitCode + ")");
		} catch (IOException e) {
			System.out.println("I/O Error occured: " + e.getMessage());
			e.printStackTrace();
			exitCode = -1;
		} catch (IllegalArgumentException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
			exitCode = -1;
		}

		System.exit((int) exitCode);
	}
}
